
# q-expansion of newform 32.10.a.e, downloaded from the LMFDB on 04 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 32
weight = 10
poly_data = [-106, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 16], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [88, 1], [702, 8], [1392, 10], [-3448, -237], [-22778, -312], [-236094, -2256], [390456, -3371], [1941840, 894], [1954806, -472], [2428608, -18808], [4304446, -56856], [-16179510, 9440], [-16115256, 27467], [-10471520, 267228], [25554126, -278936], [31549480, 533295], [12524406, -671736], [112100184, -65887], [3402736, -1910406], [-11608710, 29040], [363299424, 969956], [13741368, -1526187], [-35714550, -3553424], [925371730, 1163760], [-514728642, 4851624], [-584355408, -54366], [-980101688, 2036811], [68676998, 3010152], [562630002, -5813632], [652151808, -24678016], [2970567128, 18131217], [-1827924054, 30436704], [4002903048, -16293853], [2716284750, 24758280], [5050578000, -5216610], [8249327510, 340680], [5453028120, 28867081], [-12604281296, -32038062], [5306640102, -58668792], [-13421352840, 15504525], [-1135306450, 137241288], [-16123310208, -70642992], [9317986866, -162204816], [-22095083586, -2534584], [26656735728, 24529082], [37321714488, -92618603], [42059747136, -163255112], [-8935622760, -79192647], [-36830300962, 14040648], [-32991689574, -46488336], [-12056753376, 261164076], [30007005346, 33847344], [-40779153240, 119088735], [-55447506942, -290718208], [-4714362768, 613412394], [90658774374, -114818904], [60396320736, 433622004], [46255287214, 273720936], [-39712180470, 696880560], [13058590440, -684121801], [134656823454, 509015496], [102471864312, -354987555], [1009894416, -1153890426], [56226556762, -484234272], [57565263126, 815903784], [171333034248, 1531182467], [164965546898, 991342272], [-109886265368, -704538345], [-277742764298, 1333317864], [-141225247710, 246623680], [-227682096208, 517838178], [5430274848, 3512217900], [248413204430, 2769393960], [-519035412312, 389449567], [346350425344, 1608722400], [346432451358, -1539199128], [-315675468122, -2079859608], [-545293434942, 2133534640], [48365604922, -2170000032], [363341807000, 1772689785], [96523774430, -1802580600], [323475941280, -5127638820], [-61822205726, 2341162416], [-639888771312, 2913517222], [584052589992, -2125294401], [-1195844032974, -1526366096], [509536633322, 3524428128], [1329447439782, 2030713896], [-793115506848, -4645325116], [-464255236936, -4337678427], [-821911426752, 4350423672], [-704835658320, 2502599458], [596238902600, 6868595355], [-2150940166920, 400815165], [883831723920, -3741714474], [-552266091402, -1417775544], [-606029794902, 829508064], [1642649496456, 4648379763], [-74140006826, 15260820744], [394051830936, -7827678183], [3356068885542, 3299006408], [1505731412472, -11114291043], [-1478297044134, 4541549024], [1367225034408, -9267750113], [1333075819266, 3014929344], [1295175356424, 14885407779], [-1505296273326, -18266415744], [-3783018928688, 5039708718], [-3002950460918, -2061969552], [1632913308480, 19188114488], [-2571796292706, -2848038264], [-2841857199270, 7411464240], [-1351765919544, 16394970219], [613006176912, -8782121482], [965417897586, 24695674864], [5971631487192, -5398410063], [-1361288004240, -12033029622], [1012968515430, -38306499800], [-6511885551432, -5888275803], [1707540458510, -4099047480], [-1207294797550, -42027426960], [-4358818847682, -19745934488], [1522738259400, -54226015029], [4343425020024, -28581397619], [-5867232757386, 21435442056], [-3407427247778, 7508674152], [-4062164654240, 24821494980], [1512343261392, -62721888690], [-10341178565578, 20603416488], [-1097497053288, 16738957753], [-3518688431184, -20696956062], [2359373798688, 52136810732], [316273178670, -69716048760], [5628113486106, 3261490656], [-3952507784078, -332074128], [200122926366, 94140945384], [971370670776, 19098582117], [-13158276022890, 39655054920], [6020465173194, -29267670560], [6934510802760, -27321837605], [7289470840206, -77913894616], [10989920298000, 23284862854], [7822748242856, 38545276671], [-4546504521034, 33472503624], [1666327111408, -109419138918], [4786652763630, -80776991640], [6046438961274, -21548059424], [6244898775528, 144010878167], [967755196480, -61217633256], [18436332879430, -13239404760], [-18474398104398, -90780524352], [-2825620964232, 115919363981], [-5946526793328, -61205191530], [-12367538325240, -21889431293], [-13208734776096, 6722497236], [-25702725152688, 36135230238], [6018935880978, 6803348592], [-12718876522918, -115497637392], [-14354056874682, -191768896280], [-14842823333640, 64631111997], [23682106754394, -137893240224], [-21714596542992, 74814590010], [-7491224996728, -90427577517], [14743333604226, 26669424944], [11948368268496, 117163569870], [5340755092032, 12689564888], [595502535294, 148034693736]]
